#include <avr/io.h>
#include "uart.h"

void USARTInit(unsigned int ubrr) {
  //      
  //  UBRR = f / (16 * band)
  //   
  UBRRH = (unsigned char)(ubrr>>8);
  UBRRL = (unsigned char)(ubrr);

  //  RXC         -  
  //  |TXC        -  
  //  ||UDRE      -    
  //  |||FE       -  
  //  ||||DOR     -   
  //  |||||PE     -  
  //  ||||||U2X   -  
  //  |||||||MPCM -  
  //  ||||||||
  //  76543210
  UCSRA = 0;
  
  //  RXCIE       -    
  //  |TXCIE      -    
  //  ||UDRIE     -     
  //  |||RXEN     -  
  //  ||||TXEN    -  
  //  |||||UCSZ2  - UCSZ0:2   
  //  ||||||RXB8  - 9   
  //  |||||||TXB8 - 9   
  //  ||||||||
  //  76543210

  //        UART
  UCSRB = 1<<RXEN | 1<<TXEN;
  
  //  URSEL        -  1
  //  |UMSEL       - : 1- 0-
  //  ||UPM1       - UPM0:  1 
  //  |||UPM0      - UPM0:  1 
  //  ||||USBS     -  : 0-1, 1-2
  //  |||||UCSZ1   - UCSZ0: 2   
  //  ||||||UCSZ0  - UCSZ0: 2   
  //  |||||||UCPOL -    - 
  //  ||||||||
  //  76543210
  //  8- , 2  
  UCSRC = 1<<URSEL | 1<<USBS | 1<<UCSZ0 | 1<<UCSZ1;
}

//   
void USARTTransmitChar(char c) {
  //  ,   
  while(!( UCSRA & (1<<UDRE)));
  UDR = c;
}

//   
void USARTTransmitString(char str[]) {
  register char i = 0;
  while(str[i]) {
    USARTTransmitChar(str[i++]);
  }
}

//   
void USARTTransmitStringLn(char str[]) {
  USARTTransmitString(str);
  USARTTransmitChar((char)13);
  USARTTransmitChar((char)10);
}

//   
char USARTReceiveChar(void) {
  //  ,   
  while(!(UCSRA & (1<<RXC)));
  return UDR;
}

